<?php

namespace Drupal\vc_contacts_volunteers\Plugin\AJAXDashboardSection;

use Drupal\ajax_dashboard\Plugin\AJAXDashboardSectionBase;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\vc_contacts\Entity\VCContact;
use Drupal\views\Views;

/**
 * Class VolunteerContacts
 * @package Drupal\vc_contacts_volunteers\Plugin\AJAXDashboardSection
 *
 * @AJAXDashboardSection (
 *   id = "vc_volunteer_contacts",
 *   label = "Volunteer Contacts"
 * )
 *
 */
class VolunteerContacts extends AJAXDashboardSectionBase {

  /**
   * @param $form
   * @param FormStateInterface $form_state
   * @param null $context
   * @param null $params
   * @return array|AjaxResponse|mixed
   */
  public static function buildDashboard($form, FormStateInterface $form_state, $context = NULL, $params = NULL) {
    if ($trigger = $form_state->getTriggeringElement()) {
      $add = [
        '#type' => 'container',
        '#prefix' => '<div id="edit-dashboard" class="dashboard-wrapper">',
        '#suffix' => '</div>',
      ];
      switch ($trigger['#id']) {
        case 'vc_volunteer_contacts_primary':
          $add['value'] = self::buildPrimaryContact($trigger['#user']);
        break;
        case 'vc_volunteer_contacts_managed':
          $add['value'] = self::buildManagedContacts($trigger['#user']);
        break;
      }
      $response = new AjaxResponse();
      $response->addCommand(new ReplaceCommand('#edit-dashboard', $add));
      return $response;
    }
    else {
      if (isset($context['user'])) {
        return self::buildPrimaryContact($context['user']);
      }
      else {
        return [];
      }
    }

  }

  /**
   * @param $form
   * @param FormStateInterface $form_state
   * @param array $context
   * @param array $params
   * @return array|mixed
   */
  public static function buildSection($form, FormStateInterface $form_state, array $context = [], array $params = []) {
    if (isset($context['user'])) {
      $user = $context['user'];
      $section = [
        '#type' => 'container',
        '#cache' => [
          'contexts' => ['user', 'user.permissions']
        ]
      ];
      $section['header'] = [
        '#type' => 'html_tag',
        '#tag' => 'h3',
        '#value' => t('Volunteer Contact')
      ];
      $section['primary'] = [
        '#type' => 'button',
        '#value' => t('My Contact'),
        '#id' => 'vc_volunteer_contacts_primary',
        '#user' => $user,
        '#ajax' => [
          'callback' => get_called_class() . '::buildDashboard',
          'event' => 'click',
          'wrapper' => 'edit-dashboard',
        ],
      ];
      $u = \Drupal\user\Entity\User::load($user);
      $section['managed'] = [
        '#type' => 'button',
        '#value' => 'Managed Contacts',
        '#id' => 'vc_volunteer_contacts_managed',
        '#user' => $user,
        '#access' => $u->hasPermission('edit vccontact entities') || $u->hasPermission('volunteer edit any entities'),
        '#ajax' => [
          'callback' => get_called_class() . '::buildDashboard',
          'event' => 'click',
          'wrapper' => 'edit-dashboard',
        ],
      ];

      return $section;
    }
  }

  public static function buildPrimaryContact($user) {
    $query = \Drupal::entityQuery('vc_contact');
    $query->condition('type', 'volunteer');
    $query->condition('field_associate_user', $user);
    $profiles = VCContact::loadMultiple($query->execute());

    if (!empty($profiles)) {
      $profile = array_shift($profiles);
      /** @var $profile \Drupal\vc_contacts\Entity\VCContact */
      return [
        'header' => [
          '#type' => 'html_tag',
          '#tag' => 'h2',
          '#value' => t('Profile'),
        ],
        'edit' => Link::createFromRoute('Edit Profile',
          'entity.vc_contact.edit_form',
          ['vc_contact' => $profile->id()],
          ['attributes' => ['class' => ['btn btn-info btn-sm']],
            'query' => ['destination' => 'user/'.$user, 'associate-user' => $user]])->toRenderable(),
        'profile' => \Drupal::entityTypeManager()->getViewBuilder('vc_contact')->view($profile)
      ];
    }
    else {
      return [
        'header' => [
          '#type' => 'html_tag',
          '#tag' => 'h2',
          '#value' => t('Create your profile to get started'),
        ],
        'description' => [
          '#type' => 'html_tag',
          '#tag' => 'p',
          '#value' => t('Your profile is used for all volunteering and contact information. Additional information can be added in later sections.'),
        ],
        'button' => Link::createFromRoute('+ Create a Profile',
          'entity.vc_contact.add_form',
          ['vc_contact_type' => 'volunteer'],
          ['attributes' => ['class' => ['btn btn-sm btn-success']],
            'query' => ['destination' => 'user/'.$user, 'associate-user' => $user]])->toRenderable()
      ];
    }
  }

  public static function buildManagedContacts($user) {
    $view = Views::getView('vc_contacts_manage_contacts');
    $view->setArguments([$user, $user]);
    return $view->preview('default');
  }

}